import _slicedToArray from "@babel/runtime/helpers/esm/slicedToArray";
import _objectWithoutProperties from "@babel/runtime/helpers/esm/objectWithoutProperties";

/* eslint-disable react/no-direct-mutation-state */
import React, { useState, useEffect, useRef, forwardRef } from 'react';
import { withStyles } from '@material-ui/core/styles';
import cronstrue from 'cronstrue/i18n';
import Paper from '@material-ui/core/Paper';
import Tabs from '@material-ui/core/Tabs';
import Tab from '@material-ui/core/Tab';
import LabelBox from './labelBox';
import { metadata, loadHeaders } from './meta';
import styles from './cron-builder.styl';
var defaultValue = ['0', '0', '00', '1/1', '*', '?', '*'];

var findTab = function findTab(value) {
  var index = 0;

  if (value[1].search('/') !== -1 && value[2] === '*' && value[3] === '1/1') {
    index = 0;
  } else if (value[3] === '1/1') {
    index = 1;
  } else if (value[3].search('/') !== -1 || value[5] === 'MON-FRI') {
    index = 2;
  } else if (value[3] === '?') {
    index = 3;
  } else if (value[3].startsWith('L') || value[4] === '1/1') {
    index = 4;
  }

  return index;
};

var Cron = forwardRef(function (_ref, ref) {
  var options = _ref.options,
      locale = _ref.locale,
      onChange = _ref.onChange,
      translateFn = _ref.translateFn,
      initValue = _ref.value,
      showResultText = _ref.showResultText,
      showResultCron = _ref.showResultCron,
      classes = _ref.classes,
      props = _objectWithoutProperties(_ref, ["options", "locale", "onChange", "translateFn", "value", "showResultText", "showResultCron", "classes"]);

  var _useState = useState(loadHeaders(options)),
      _useState2 = _slicedToArray(_useState, 1),
      headers = _useState2[0];

  var _useState3 = useState(initValue),
      _useState4 = _slicedToArray(_useState3, 2),
      value = _useState4[0],
      setValue = _useState4[1];

  var _useState5 = useState(findTab(value)),
      _useState6 = _slicedToArray(_useState5, 2),
      tabValue = _useState6[0],
      setTabValue = _useState6[1];

  var _useState7 = useState(100),
      _useState8 = _slicedToArray(_useState7, 2),
      tabWidth = _useState8[0],
      setTabWidth = _useState8[1];

  var tabRef = useRef(null);
  useEffect(function () {
    onChange(value.toString().replace(/,/g, ' ').replace(/!/g, ','));

    if (translateFn && !locale) {
      console.log('Warning !!! locale not set while using translateFn');
    }

    setTabWidth(tabRef.current.offsetWidth / headers.length);
    window.addEventListener('resize', handleResize);
    return function () {
      return window.removeEventListener('resize', handleResize);
    };
  }, []);
  useEffect(function () {
    onChange(value.toString().replace(/,/g, ' ').replace(/!/g, ','));
  }, [value]);
  useEffect(function () {
    if (value !== initValue) setValue(JSON.parse(JSON.stringify(getMetadata().initialCron)));
  }, [tabValue]);

  var handleResize = function handleResize() {
    setTabWidth(tabRef.current.offsetWidth / headers.length);
  };

  var onValueChange = function onValueChange(value) {
    setValue(value && value.length ? value : defaultValue);
  };

  var getVal = function getVal() {
    var val = cronstrue.toString(value.toString().replace(/,/g, ' ').replace(/!/g, ','), {
      locale: locale
    });
    return val.search('undefined') === -1 ? val : '-';
  };

  var getMetadata = function getMetadata() {
    var selectedMetaData = metadata.find(function (data) {
      return data.component.muiName === "".concat(headers[tabValue], "Cron");
    });

    if (!selectedMetaData) {
      selectedMetaData = metadata[tabValue];
    }

    if (!selectedMetaData) {
      throw new Error('Value does not match any available headers.');
    }

    return selectedMetaData;
  };

  var getComponent = function getComponent() {
    var CronComponent = getMetadata().component;
    return /*#__PURE__*/React.createElement(CronComponent, {
      translate: translate,
      value: value,
      onChange: onValueChange
    });
  };

  var translate = function translate(key) {
    var translatedText = key;

    if (translateFn) {
      translatedText = translateFn(key);

      if (typeof translatedText !== 'string') {
        throw new Error('translateFn expects a string translation');
      }
    }

    return translatedText;
  };

  var getHeaders = function getHeaders() {
    var style = {
      width: "".concat(tabWidth, "px !important"),
      maxWidth: "".concat(tabWidth, "px"),
      minWidth: "".concat(tabWidth, "px")
    };
    return headers.map(function (d, index) {
      return /*#__PURE__*/React.createElement(Tab, {
        key: index,
        label: translate(d),
        style: style
      });
    });
  };

  return /*#__PURE__*/React.createElement(Paper, Object.assign({
    ref: ref
  }, props), /*#__PURE__*/React.createElement(Tabs, {
    ref: tabRef,
    value: tabValue,
    indicatorColor: "primary",
    textColor: "primary",
    fillWidth: true,
    onChange: function onChange(ev, value) {
      setTabValue(value);
    },
    "aria-label": "disabled tabs example"
  }, getHeaders()), /*#__PURE__*/React.createElement("div", {
    className: "cron_builder_bordering"
  }, getComponent()), showResultText && /*#__PURE__*/React.createElement(LabelBox, null, getVal()), showResultCron && /*#__PURE__*/React.createElement(LabelBox, null, value.toString().replace(/,/g, ' ').replace(/!/g, ',')));
});
Cron.defaultProps = {
  locale: 'en',
  value: defaultValue,
  onChange: function onChange(value) {
    console.log(value);
  },
  showResultText: true,
  showResultCron: true
};
export default withStyles(styles)(Cron);